@@ -1,5 +1,7 @@
 from kivy import Config
-import os, darkdetect
+import os, darkdetect, threading
+
+from kivymd.uix.button import MDFlatButton
 
 Config.set('graphics', 'resizable', 0)
 Config.set('input', 'mouse', 'mouse, multitouch_on_demand')
@@ -12,10 +14,11 @@ from kivy.uix.settings import ContentPanel
 from kivymd.app import MDApp
 from screeninfo import get_monitors
 from kivy.properties import StringProperty
+from kivymd.uix.snackbar import Snackbar
+from kivymd.toast import toast
 
 
 def getSpec(name: str, filename: str = 'main.py', icon: str = '', path: str = os.getcwd()):
-
     path = path.replace("\\", r"\\")
 
     with open(filename.replace('.py', '.spec'), 'w') as f:
@@ -70,24 +73,34 @@ coll = COLLECT(exe, Tree('{path}'),
                name='{name}')""")
 
 
-class Kivy4(MDApp):
+def thread(func):
+    def inner(*args, **kwargs):
+        threading.Thread(target=lambda: func(*args, **kwargs), daemon=True).start()
 
+
+    return inner
+
+
+class Kivy4(MDApp):
     dark_mode_icon = StringProperty('')
 
-    def __init__(self, string: str = '', app_name: str = '', list_of_files: list = None,
-                 screen_size=None, min_x: int = 0.1, min_y: int = 0.1, center: bool = True,
+    def __init__(self, string: str = '', app_name: str = '', dict_of_files: dict = None,
+                 screen_size=None, minimum=None, center: bool = True,
                  sun_icon: str = 'white-balance-sunny', moon_icon: str = 'weather-night',
-                 main_color: str = 'Blue', icon: str = '', toolbar = False, app_data: bool = True,
-                 custom_classes: bool = True, disable_x: bool = False, **kwargs):
+                 main_color: str = 'Blue', icon: str = '', toolbar=False, app_data: bool = True,
+                 custom_classes: bool = True, disable_x: bool = False, pre_string: str = '', **kwargs):
 
         super().__init__(**kwargs)
 
+        if minimum is None:
+            minimum = [0.1, 0.1]
+
         self.app_name = app_name
 
         if app_data:
             app_data_path = os.getenv('APPDATA') + '/' + app_name
             self.appdata_path = app_data_path
-            self.create_files(list_of_files)
+            self.create_files(dict_of_files)
 
             self.moon_icon = moon_icon
             self.sun_icon = sun_icon
@@ -101,21 +114,38 @@ class Kivy4(MDApp):
         self.width = screen.width
         self.height = screen.height
 
-        self.screen_positions(screen_size, min_x, min_y, center)
+        self.screen_positions(screen_size, minimum, center)
 
-        self.builder_string = ''
+        self.builder_string = pre_string
 
         if custom_classes:
             self.builder_string += self.custom_classes()
 
         if toolbar:
             self.builder_string += self.getToolbar(toolbar)
-            string = '\n            '.join(string.split('\n'))
 
-        self.builder_string += string
+            z = string.split('\n')
+            for x in z:
+                if 'x,y=' in x.replace(' ', ''):
+                    self.builder_string += '\n            ' + self.x_y(x)
+
+                else:
+                    self.builder_string += '\n            ' + x
 
         self.run()
 
+    @staticmethod
+    def x_y(x_y):
+        tabs = ""
+        for u in x_y:
+            if u == " ":
+                tabs += " "
+            else:
+                break
+
+        x, y = eval(x_y.split("=")[1])
+        return f"{tabs}pos_hint: {{'center_x': {x}, 'center_y': {y}}}"
+
     def build(self):
         self.use_kivy_settings = False
         self.settings_cls = ContentPanel
@@ -125,14 +155,13 @@ class Kivy4(MDApp):
         Window.bind(on_drop_file=self.on_drop_file)
 
         self.Build()
-
         return Builder.load_string(self.builder_string)
 
     def Build(self):
         pass
 
-    def screen_positions(self, screen_size, min_x, min_y, center):
-
+    def screen_positions(self, screen_size, minimum, center):
+        min_x, min_y = minimum
         if screen_size is None:
             x, y = 0.6, 0.6
 
@@ -144,44 +173,51 @@ class Kivy4(MDApp):
             Window.minimum_height = self.height * min_y
             Window.minimum_width = self.width * min_x
 
+
         else:
             Window.size = (x, y)
             Window.minimum_height = min_y
             Window.minimum_width = min_x
 
+            if center:
+                Window.left = (self.width - x) / 2
+                Window.top = (self.height - y) / 2
+                return
+
+            else:
+                return
+
         if center:
             Window.left = (self.width - (self.width * x)) / 2
             Window.top = (self.height - (self.height * y)) / 2
 
     def create_files(self, list_of_files):
-
         try:
             if not os.path.isdir(self.appdata_path):
                 os.mkdir(self.appdata_path)
 
             if list_of_files:
                 for file in list_of_files:
-                    self.setFile(file + '.txt', list_of_files[file], check_if_file_exist=True)
+                    self.setFile(file, list_of_files[file])
 
         except Exception as e:
             return e
 
-    def setFile(self, file, value, check_if_file_exist=False):
+    def setFile(self, file, value, extension='.txt'):
 
-        path_to_create = f'{self.appdata_path}/{file}'
+        path_to_create = f'{self.appdata_path}/{file}{extension}'
 
         try:
-            if not check_if_file_exist or not os.path.isfile(path_to_create):
-                with open(path_to_create, 'w') as f:
-                    f.write(value)
+            with open(path_to_create, 'w') as f:
+                f.write(value)
 
         except Exception as e:
             print(e)
             return e
 
-    def getFile(self, file, default=None, create_file_if_not_exist=False):
+    def getFile(self, file, default=None, create_file_if_not_exist=False, extension='.txt'):
 
-        path_of_file = f'{self.appdata_path}/{file}'
+        path_of_file = f'{self.appdata_path}/{file}{extension}'
 
         try:
             with open(path_of_file, 'r') as f:
@@ -193,6 +229,9 @@ class Kivy4(MDApp):
                     f.write(default)
                     return default
 
+            else:
+                return default
+
         except Exception as e:
             print(e)
             return default
@@ -218,13 +257,27 @@ class Kivy4(MDApp):
         except AttributeError:
             return False
 
-    def setDarkMode(self, value=None, filename='dark mode.txt'):
+    def setDarkMode(self, value=None, filename='dark mode'):
         if value is None:
             value = darkdetect.theme()
 
         self.setFile(filename, value)
         self.setDarkModeIcon(value)
 
+    @staticmethod
+    def toast(text, duration=2.5):
+        toast(text=text, duration=duration)
+
+    @staticmethod
+    def snack(text, button_text=None, func=None):
+        snack = Snackbar(text=text)
+
+        if func and button_text:
+            snack.buttons = [MDFlatButton(text=f"[color=#1aaaba]{button_text}[/color]",
+                                          on_release=func, ), ]
+
+        snack.open()
+
     def reverseDarkMode(self, filename: str = 'dark mode.txt'):
         try:
             with open(self.appdata_path + '/' + filename, 'r') as f:
@@ -280,10 +333,10 @@ Screen:
         id: toolbar
         pos_hint: {{"top": 1}}
         elevation: 10
-        title: "{משצק}"
+        title: "{name}"
         right_action_items: {right_icons}
         left_action_items: {left_icons}
-    
+
     MDNavigationLayout:
         x: toolbar.height
         ScreenManager:
@@ -306,12 +359,17 @@ Screen:
     size_hint: None, None
     size: dp(48), dp(48)
 
-<Btn@MDFillRoundFlatIconButton>:
+<Btn@MDFillRoundFlatButton>:
     text: ""
-    
-    
+
+<BtnIcon@MDFillRoundFlatIconButton>:
+    text: ""
+
 <Img@Image>:    
     allow_stretch: True
+
+<CircleIcon@MDFloatingActionButton>:
+    md_bg_color: app.theme_cls.primary_color
 '''
 
     @staticmethod
